//   Copyright 2013 Christian Gjerde Jensen

//   Licensed under the Apache License, Version 2.0 (the "License");
//   you may not use this file except in compliance with the License.
//   You may obtain a copy of the License at

//       http://www.apache.org/licenses/LICENSE-2.0

//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS,
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and
//   limitations under the License.

////////////////
// VIEW MODEL //
////////////////

var viewModel = function() {
  var news = ko.observableArray([]);
  var cPage = ko.observable("news");
  var categories = ko.observableArray([]);
  var supportsGeolocation = ko.observable(true);
  var enableLocationNews = ko.observable(true);
  var enableTimeNews = ko.observable(true);

  //Load preferences from HTML5 Web Storage
  if (localStorage.getItem("enableLocationNews") !== null) {
    enableLocationNews = ko.observable(JSON.parse(localStorage.getItem("enableLocationNews")));
  }
  if (localStorage.getItem("enableTimeNews") !== null) {
    enableTimeNews = ko.observable(JSON.parse(localStorage.getItem("enableTimeNews")));
  }
  if (localStorage.getItem("categories") !== null) {
    var savedCats = JSON.parse(localStorage.getItem("categories"));
    for (var i in savedCats) {
      categories.push({name: savedCats[i].name, weight: ko.observable(savedCats[i].weight)});
    }
  } else {
    var defaultCats = ["Innenriks", "Utenriks", "Politikk", "Sport", "Bil", "Finans", "Mote", "Teknologi", "Dataspill"];
    for (var i in defaultCats) {
      categories.push({name: defaultCats[i], weight: ko.observable(0)});
    }
  }

  //Helper functions for the preference lists in the configuration panel
  var notAllSelected = ko.computed(function() {
    return ko.utils.arrayFirst(categories(), function(cat) {
      return cat.weight() < 0.5;
    });
  });
  var notNoSelected = ko.computed(function() {
    return ko.utils.arrayFirst(categories(), function(cat) {
      return cat.weight() >= 0.5;
    });
  });

  function addNews(category, articles) {
    news.push({category: category, articles: articles});
  }

  //Add a new category or increase the weight of an existing category.
  //Weight should be a value between 0 and 1, where 1 is used
  function addCategory(category, weight) {
    var index = -1;
    ko.utils.arrayFirst(categories(), function(cat) {
      if (category === cat.name) {
        index = categories.indexOf(cat);
      }
    });

    //When the weight of a category is increased, decay all other weights.
    if (weight < 1 && index > 0) {
      var i = 0;
      ko.utils.arrayForEach(categories(), function (cat) {
        if (categories()[i].weight() > (weight * 0.1)) {
          categories()[i].weight(categories()[i].weight() - (weight * 0.1));
        } else {
          categories()[i].weight(0);
        }
        i++;
      });
    }

    if (index < 0) {
      categories.push({name: category, weight: ko.observable(weight)});
    } else {
      categories()[index].weight(categories()[index].weight() + weight);
    }
    localStorage.setItem("categories", ko.toJSON(categories));
  }

  //Remove a news category from the preferred news of the user
  function removeCategory(category) {
    categories()[categories.indexOf(category)].weight(0);
    localStorage.setItem("categories", ko.toJSON(categories));
  }

  //Sammy routing
  Sammy(function() {
    this.get('#/news', function() {
      news.removeAll();
      cPage("news");
      ko.utils.arrayForEach(categories(), function (cat) {
        newsHandler.getNews(cat.name, 4);
      });
    });

    this.get('#/news/personal', function() {
      news.removeAll();
      cPage("news");

      if (context.getLocation() && enableLocationNews()) {
        newsHandler.getNews(context.getLocation(), 4);
      }
      if (enableTimeNews()) {
        newsHandler.getNews(context.getTime(), 4);
      }
      ko.utils.arrayForEach(categories(), function (cat) {
        if (cat.weight() >= 0.5) {
          newsHandler.getNews(cat.name, 4);
        }
      });
    });

    this.get('#/news/cat/:cat', function() {
      news.removeAll();
      cPage("news");
      newsHandler.getNews(this.params.cat, 16);
    });

    this.get('#/config', function() {
      cPage("config");
    });

    this.get('', function() { this.app.runRoute('get', '#/news'); });
  }).run();

  return {
    news: news,
    cPage: cPage,
    categories: categories,
    supportsGeolocation: supportsGeolocation,
    enableLocationNews: enableLocationNews,
    enableTimeNews: enableTimeNews,
    notAllSelected: notAllSelected,
    notNoSelected: notNoSelected,
    addNews: addNews,
    addCategory: addCategory,
    removeCategory: removeCategory
  };
}();

//Initialize view model and persist preferences
$(document).ready(function () {
  ko.applyBindings(viewModel);

  viewModel.enableLocationNews.subscribe(function(val) {
    localStorage.setItem("enableLocationNews", val);
  });

  viewModel.enableTimeNews.subscribe(function(val) {
    localStorage.setItem("enableTimeNews", val);
  });
});